﻿Imports System.Xml
Imports System.Threading
Imports System.Data.OleDb

Imports MySql.Data.MySqlClient

Imports CrystalDecisions.CrystalReports.Engine
Imports CrystalDecisions.Windows.Forms

Module Database
    Public Conn As MySqlConnection
    Public DbAdapter As New MySqlDataAdapter
    Public DbDataReader As MySqlDataReader
    Public DbCmd As New MySqlCommand
    Public DtResult As DataTable

    ''' <summary>
    ''' Sets the connection to the database.
    ''' </summary>
    ''' <param name="DbHostName">Set the host name of the server.</param>
    ''' <param name="DbUserName">Set the UserEmail to the server.</param>
    ''' <param name="DbPassword">Set the password to the server.</param>
    ''' <param name="DbName">Set the database name to be used in the program.</param>
    ''' <remarks>Sets the connection to the database.</remarks>
    Public Sub SetDbConnectionString(ByVal DbHostName As String, ByVal DbUserName As String, ByVal DbPassword As String, ByVal DbName As String)

        Dim DbSetup As String = String.Format("server={0}; user id={1}; password={2}; database={3};", DbHostName, DbUserName, DbPassword, DbName)

        Conn = New MySqlConnection(DbSetup)
    End Sub

    ''' <summary>
    ''' Toggles the connection of the database connection. If it is open then close it otherwise open it if it closed.
    ''' </summary>
    ''' <remarks>Toggles the connection of the database connection.</remarks>
    Public Sub SetDbConnection()
        Try
            If Conn.State = ConnectionState.Open Then Conn.Close()
            Conn.Open()
        Catch ex As Exception
            'MessageBox.Show(ex.Message, PROGRAM_NAME, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
        End Try
    End Sub

    ''' <summary>
    ''' Executes a query to the database.
    ''' </summary>
    ''' <param name="Query">The query that will be exceuted.</param>
    ''' <param name="DtgResult">The datagridview where the resut will take place.</param>
    ''' <remarks>Excecutes a query in the database.</remarks>
    Public Sub Query(ByVal Query As String, ByVal DtgResult As DataGridView)
        Try
            Dim QueryResultCount As Integer = 0
            DtResult = New DataTable

            SetDbConnection()
            With DbCmd
                .Connection = Conn
                .CommandText = Query
            End With

            DbAdapter.SelectCommand = DbCmd
            DbAdapter.Fill(DtResult)

            DtgResult.DataSource = DtResult

            With DtgResult
                .AutoSizeColumnsMode = DataGridViewAutoSizeColumnsMode.Fill
                .AutoSizeRowsMode = DataGridViewAutoSizeRowsMode.AllCells
                .ColumnHeadersDefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleCenter
                .RowHeadersVisible = False
                .SelectionMode = DataGridViewSelectionMode.FullRowSelect
                For Each Col As DataGridViewColumn In DtgResult.Columns
                    If Col.HeaderCell.Value.Contains("id") Then
                        Col.Visible = False
                    End If
                Next
            End With
        Catch ex As Exception
            'MessageBox.Show(ex.Message, PROGRAM_NAME, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
        End Try
    End Sub

    ''' <summary>
    ''' Executes a query to the database and initialize a messagebox when the query is executed successfully! 
    ''' </summary>
    ''' <param name="Message">The message that wil be initialize after the query is successfully executed</param>
    ''' <param name="Query">The query that will be executed in the database.</param>
    ''' <remarks>Executes a query and initialize a messagebox.</remarks>
    Public Sub Query(ByVal Message As String, ByVal Query As String)
        Try

            Dim QueryResultCount As Integer = 0
            DtResult = New DataTable

            SetDbConnection()

            If Query IsNot String.Empty Then
                With DbCmd
                    .Connection = Conn
                    .CommandText = Query
                End With

                QueryResultCount = DbCmd.ExecuteNonQuery()
                If QueryResultCount < 0 Then
                    MessageBox.Show("Unable to load data!", PROGRAM_NAME, MessageBoxButtons.OK, MessageBoxIcon.Warning)
                Else
                    MessageBox.Show(Message, PROGRAM_NAME, MessageBoxButtons.OK, MessageBoxIcon.Information)
                End If
            End If

        Catch ex As Exception

            'MessageBox.Show(ex.Message, PROGRAM_NAME, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)

        End Try
    End Sub

    ''' <summary>
    ''' Executes a query directly to the database.
    ''' </summary>
    ''' <param name="Query">The query that will be executed in the database.</param>
    ''' <remarks>Executes a query and initialize a messagebox.</remarks>
    Public Sub Query(ByVal Query As String)
        Try

            Dim QueryResultCount As Integer = 0
            DtResult = New DataTable

            SetDbConnection()
            With DbCmd
                .Connection = Conn
                .CommandText = Query
            End With

            QueryResultCount = DbCmd.ExecuteNonQuery()
            If QueryResultCount < 0 Then
                MessageBox.Show("Unable to load data!", PROGRAM_NAME, MessageBoxButtons.OK, MessageBoxIcon.Warning)
            End If

        Catch ex As Exception

            'MessageBox.Show(ex.Message, PROGRAM_NAME, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)

        End Try
    End Sub


    ''' <summary>
    ''' Executes a query for the tables only to the database.
    ''' </summary>
    ''' <param name="Query">The query that will be exceuted.</param>
    ''' <param name="DtgResult">The datagridview where the result will take place.</param>
    ''' <remarks>Excecutes a query for showing tables only in the database.</remarks>
    Public Sub EditTableQuery(ByVal Query As String, ByVal DtgResult As DataGridView)
        Try
            Dim QueryResultCount As Integer = 0
            DtResult = New DataTable

            SetDbConnection()
            With DbCmd
                .Connection = Conn
                .CommandText = Query
            End With

            DbAdapter.SelectCommand = DbCmd
            DbAdapter.Fill(DtResult)

            Try
                Dim DataTypeArray = Nothing
                For Each Row As DataRow In DtResult.Rows
                    DataTypeArray = Row.Item(1).ToString().Replace("(", " ").Replace(")", "").Split(" ")

                    If Row.Item(1).ToString.Contains("(") Then
                        If DataTypeArray(0) = "int" Then
                            DtgResult.Rows.Add(Row.Item(0), "INT", DataTypeArray(1))
                        ElseIf DataTypeArray(0) = "varchar" Then
                            DtgResult.Rows.Add(Row.Item(0), "VARCHAR", DataTypeArray(1))
                        ElseIf DataTypeArray(0) = "text" Then
                            DtgResult.Rows.Add(Row.Item(0), "TEXT", DataTypeArray(1))
                        ElseIf DataTypeArray(0) = "date" Then
                            DtgResult.Rows.Add(Row.Item(0), "DATE", DataTypeArray(1))
                        ElseIf DataTypeArray(0) = "double" Then
                            DtgResult.Rows.Add(Row.Item(0), "DOUBLE", DataTypeArray(1))
                        End If
                    Else
                        DtgResult.Rows.Add(Row.Item(0), "", DataTypeArray(1))
                    End If

                Next
            Catch ex As MySqlException
                'MessageBox.Show(ex.Message, PROGRAM_NAME, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            End Try



        Catch ex As Exception
            'MessageBox.Show(ex.Message, PROGRAM_NAME, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
        End Try
    End Sub

    ''' <summary>
    ''' Executes a query for the editing of tables to the database.
    ''' </summary>
    ''' <param name="Query">The query that will be exceuted.</param>
    ''' <param name="DtgResult">The datagridview where the result will take place.</param>
    ''' <remarks>Excecutes a query for showing tables only in the database.</remarks>
    Public Sub TableQuery(ByVal Query As String, ByVal DtgResult As DataGridView)
        Try
            Dim QueryResultCount As Integer = 0
            DtResult = New DataTable

            SetDbConnection()
            With DbCmd
                .Connection = Conn
                .CommandText = Query
            End With

            DbAdapter.SelectCommand = DbCmd
            DbAdapter.Fill(DtResult)

            DtgResult.DataSource = DtResult

            With DtgResult
                .AutoSizeColumnsMode = DataGridViewAutoSizeColumnsMode.Fill
                .AutoSizeRowsMode = DataGridViewAutoSizeRowsMode.AllCells
                .ColumnHeadersDefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleCenter
                .RowHeadersVisible = False
                .SelectionMode = DataGridViewSelectionMode.FullRowSelect
                For Each Col As DataGridViewColumn In DtgResult.Columns
                    If Col.HeaderCell.Value.Contains("id") Then
                        Col.Visible = False
                    End If
                Next
            End With
        Catch ex As Exception
            'MessageBox.Show(ex.Message, PROGRAM_NAME, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
        End Try
    End Sub

    ''' <summary>
    ''' Query's to the database that returns only a single value.
    ''' </summary>
    ''' <param name="Query">The query that will be executed.</param>
    ''' <returns>The single result form the query.</returns>
    ''' <remarks>Query's to the database that returns only a single value.</remarks>
    Public Function GetSingleValue(ByVal Query As String) As String
        Dim Result As String = New String("")

        Try
            SetDbConnection()
            DbCmd = New MySqlCommand(Query, Conn)

            DbDataReader = DbCmd.ExecuteReader
            While DbDataReader.Read
                If Not IsDBNull(DbDataReader(0)) Then
                    Result = DbDataReader(0)
                End If
            End While
        Catch ex As Exception
            'MessageBox.Show(ex.Message, PROGRAM_NAME, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
        End Try

        Return Result
    End Function

    ''' <summary>
    ''' Sets an autocompletion to a textbox from a database.
    ''' </summary>
    ''' <param name="SqlQuery">The query for getting the specific column's data.</param>
    ''' <param name="TargetTextBox">The text where the autocompletion will put into action.</param>
    ''' <remarks>Adds an autocompletion to a textbox from a database.</remarks>
    Public Sub SetAutoCompletion(ByVal SqlQuery As String, ByVal TargetTextBox As TextBox)
        Dim Result As New AutoCompleteStringCollection

        SetDbConnection()

        Try
            DbCmd = New MySqlCommand(SqlQuery, Conn)

            DbDataReader = DbCmd.ExecuteReader
            While DbDataReader.Read()
                Result.Add(DbDataReader.GetValue(0))
            End While
            DbDataReader.Close()
        Catch ex As Exception
            'MessageBox.Show(ex.Message, PROGRAM_NAME, MessageBoxButtons.OK, MessageBoxIcon.Warning)
        End Try

        TargetTextBox.AutoCompleteMode = AutoCompleteMode.SuggestAppend
        TargetTextBox.AutoCompleteSource = AutoCompleteSource.CustomSource
        TargetTextBox.AutoCompleteCustomSource = Result
    End Sub

    ''' <summary>
    ''' Get all the records of a specific column of a table by using an id.
    ''' </summary>
    ''' <param name="SqlQuery">The query to get all the cols</param>
    ''' <param name="TargetControl">The combobox where the output will go.</param>
    ''' <remarks>Get all the records in a table.</remarks>
    Public Sub SetAutoCompletion(ByVal SqlQuery As String, ByVal TargetControl As ComboBox)
        TargetControl.Items.Clear()

        Try
            SetDbConnection()

            DbCmd = New MySqlCommand(SqlQuery, Conn)
            DbDataReader = DbCmd.ExecuteReader

            While DbDataReader.Read()
                TargetControl.Items.Add(DbDataReader.GetValue(0))
            End While
            TargetControl.AutoCompleteSource = AutoCompleteSource.ListItems
            TargetControl.Text = TargetControl.Items(0)

        Catch ex As Exception
            'MessageBox.Show(ex.Message, PROGRAM_NAME, MessageBoxButtons.OK, MessageBoxIcon.Warning)
        End Try

        DbDataReader.Close()
    End Sub

    ''' <summary>
    ''' Displays a report to a report viewer.
    ''' </summary>
    ''' <param name="Query">Teh query that will be executed in the report</param>
    ''' <param name="FilePath">The file path for the template of the report to be used.</param>
    ''' <param name="ReportViewer">The report viewer where all the data will take place.</param>
    ''' <remarks>Displays a report to a report viewer.</remarks>
    Public Sub PrintReport(ByVal Query As String, ByVal FilePath As String, ByVal ReportViewer As CrystalReportViewer)
        Dim Report As New ReportDocument
        DtResult = New DataTable

        SetDbConnection()
        Try

            Dim QueryResultCount As Integer = 0

            SetDbConnection()
            With DbCmd
                .Connection = Conn
                .CommandText = Query
            End With

            DbAdapter.SelectCommand = DbCmd
            DbAdapter.Fill(DtResult)
        Catch ex As Exception

            'MessageBox.Show(ex.Message, PROGRAM_NAME, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
        End Try

        Report.Load(Application.StartupPath & FilePath)
        Report.SetDataSource(DtResult)
        ReportViewer.ReportSource = Report
        ReportViewer.Refresh()
    End Sub
End Module
